/***************************************************************************
                          klprfax.cpp  -  description
                             -------------------
    begin                : Thu May 24 23:17:55 'Europe/Amsterdam' 2001
    copyright            : (C) 2001 by Hans Dijkema
    email                : hdijkema@hum.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "klprfax.h"

#include <kmessagebox.h>
#include <kaboutapplication.h>

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <wait.h>

#include <klocale.h>

Klprfax::Klprfax(KApplication *_app,KConfigBase *cnf,QCString faxF,QWidget *parent, const char *name) : QWidget(parent, name)
{
AddressBook::ErrorCode err;
AddressBook   *bk;
QListView     *start;
int            n,i;
QString        name,fax;
QString        nil="";

   app=_app;

   fatal=false;
   findEFax();

   // Dialog

   dlg=new QGrid(4,QGrid::Vertical,this);
   dlg->setMargin(10);
   dlg->setSpacing(10);

   // Fax numbers

   start=names=new QListView(dlg);
   names->addColumn(i18n("Name"));
   names->addColumn(i18n("Fax Nr."));

   // Headers

   edits=new QGrid(2,dlg);
   edits->setSpacing(10);
   edits->setMargin(10);
   edits->setFrameStyle(QFrame::Box|QFrame::Raised);

   lfrom=new QLabel(edits);
   lfrom->setText(i18n("your header :"));
   from=new QLineEdit(edits);
   from->setMinimumWidth(200);

   lfromFax=new QLabel(edits);
   lfromFax->setText(i18n("your fax :"));
   fromFax=new QLineEdit(edits);

   conf=cnf;
   conf->setGroup("klprfax");
   {QString ft,fft;
     ft=conf->readEntry("from",nil);
     fft=conf->readEntry("fromFax",nil);
     from->setText(ft);
     fromFax->setText(fft);
   }

   lprefix=new QLabel(edits);
   lprefix->setText(i18n("Dial prefix :"));
   prefix=new QLineEdit(edits);
   prefix->setText(conf->readEntry("dialprefix",nil));

   // Progress bar

   progr=new QGrid(2,dlg);
   progr->setSpacing(10);

   doing=new QLabel(progr);
   doing->setText(i18n("doing nothing"));
   progress=new QProgressBar(progr);
   progress->setTotalSteps(4);
   progress->reset();

   // Buttons

   buttons=new QGrid(4,dlg);
   buttons->setSpacing(10);

   about=new QPushButton(buttons);about->setText(i18n("&About"));
   configure=new QPushButton(buttons);configure->setText(i18n("&Save config"));
   noFax=new QPushButton(buttons);noFax->setText(i18n("&Done"));
   goFax=new QPushButton(buttons);goFax->setText(i18n("&Fax document"));

   connect(about,SIGNAL(clicked()),SLOT(doAbout()) );
   connect(configure, SIGNAL(clicked()), SLOT(setConfig()) );
   connect(noFax, SIGNAL(clicked()), SLOT(noFaxing()) );
   connect(goFax, SIGNAL(clicked()), SLOT(goFaxing()) );

   // Search all entries with fax numbers

   kab=new KabAPI;
   err=kab->init();
   bk=kab->addressbook();
   n=bk->noOfEntries();

   for(i=0;i<n;i++) {KabKey k;
                     AddressBook::Entry e;
     err=bk->getKey(i,k);
     err=bk->getEntry(k,e);
     if (NameFax(e,name,fax)) {
        new QListViewItem(names,name,fax);
     }
   }

   // adjust the dialog size

   dlg->adjustSize();
   setFixedSize(dlg->width(),dlg->height());

   // Read the fax to fax.

   {QString cwd;
    char line[4096];
      getcwd(line,4096);
      cwd.sprintf("%s",line);
      faxFile=faxF;faxFile=cwd+"/"+faxFile;
      {FILE *f=fopen(faxFile.latin1(),"rb");
          if (f==NULL) { faxFile=faxF; }
          f=fopen(faxFile.latin1(),"rb");
          if (f==NULL) {
            KMessageBox::error(this,i18n("Can't open file to fax"));
            fatal=true;
          }
      }
   }
}

Klprfax::~Klprfax()
{
   delete names;
   delete goFax;
   delete noFax;
   delete from;
   delete fromFax;
   delete progress;
   delete doing;
}

/** No descriptions */
bool Klprfax::NameFax(AddressBook::Entry & e,QString & name, QString & fax)
{
bool isFax=false,ok;
unsigned int n,i;
QString nil="";
AddressBook::Telephone faxEnum=AddressBook::Fax; // 3;  // AddressBook::Telephone::Fax

   n=e.telephone.count();
   for(i=0;i<n && !isFax;i+=2) {QString type;
                                 int _type;
      type=e.telephone[i];
      _type=type.toInt(&ok,10);
      if (ok) {
        if ((_type+1)==faxEnum) {
          fax=e.telephone[i+1].stripWhiteSpace();
          if (fax!="") { isFax=true; }
        }
      }
   }

   if (isFax) {
     name=e.fn.stripWhiteSpace();
     if (name=="") {QString fn,mn,ln;
       fn=e.firstname.stripWhiteSpace();
       mn=e.middlename.stripWhiteSpace();
       ln=e.lastname.stripWhiteSpace();
       if (mn=="") { name=fn+" "+ln; }
       else { name=fn+" "+mn+" "+ln; }
     }
   }

return isFax;
}
/** No descriptions */
void Klprfax::noFaxing()
{
   close();
}
/** No descriptions */
/** No descriptions */
void Klprfax::setConfig(void)
{
   conf->setGroup("klprfax");
   conf->writeEntry("from",from->text());
   conf->writeEntry("fromFax",fromFax->text());
   conf->writeEntry("dialprefix",prefix->text().stripWhiteSpace());
}
/** No descriptions */
void Klprfax::goFaxing()
{
QListViewItem *it;
QString tel;
int     forkResult;
QString device;

  {FILE *dev=fopen("/etc/klprfax","rb");
   char line[1024];
     if (dev==NULL) {
        KMessageBox::error(this,i18n("No modem device configured, run klprfax as root"));
        return;
     }
     fgets(line,1024,dev);
     line[strlen(line)-1]='\0';
     device=line;
     fclose(dev);
  }

  it=names->selectedItem();
  if (it==0) {
    KMessageBox::error(this,i18n("Select a fax number"));
    return;
  }

  tel=it->text(1);  // get fax number
  tel=prefix->text().stripWhiteSpace()+tel;   // add the prefix

  // Now start the faxing!!!

  {QString script,cmd,dir,filetofax,logfile="log";
   int     k;

     script.sprintf("/tmp/fax.%d",getpid());
     dir.sprintf("/tmp/faxd.%d",getpid());

     doing->setText(i18n("preparing fax command..."));
     progress->setProgress(1);

     FILE *fin=fopen(efax,"rb");
     FILE *fout=fopen(script,"wb");
     char line[4096];

     while(fgets(line,4096,fin)!=NULL) {
        if (strncmp(line,"NAME=",5)==0) {
          fprintf(fout,"NAME=\"%s\"\n",from->text().latin1());
        }
        else if (strncmp(line,"FROM=",5)==0) {
          fprintf(fout,"FROM=\"%s\"\n",fromFax->text().latin1());
        }
        else if (strncmp(line,"FAILRETRIES=",12)==0 || strncmp(line,"BUSYRETRIES=",12)==0) {
          // skip line
        }
        else if (strncmp(line,"DEV=",4)==0) {
          fprintf(fout,"DEV=\"%s\"\n",device.latin1());
        }
        else {
          fputs(line,fout);
        }
     }

     fclose(fin);fclose(fout);

     doing->setText(i18n("copying input file..."));
     progress->setProgress(2);

     mkdir(dir.latin1(),S_IRWXU);
     if (chdir(dir.latin1())==-1) {QString msg;
        msg=i18n("Can't change directory to :")+dir;
        KMessageBox::error(this,msg);
        return;
     }

     {FILE *fin=fopen(faxFile.latin1(),"rb"),
           *fout=fopen("faxfile","wb");
           int fh=fileno(fin);
           char buf[8192];
           int bytes;
           while((bytes=read(fh,buf,8192))!=0) {
              fwrite(buf,bytes,1,fout);
              app->processEvents(50);
           }
           fclose(fin);
           fclose(fout);
           filetofax="faxfile";
     }

     doing->setText(i18n("efax is running..."));
     progress->setProgress(3);

     cmd.sprintf("sh %s send \"%s\" \"%s\" >%s 2>&1",script.latin1(),tel.latin1(),filetofax.latin1(),logfile.latin1());

     goFax->setEnabled(false);
     noFax->setEnabled(false);

     if ((forkResult=fork())==0) {
       k=system(cmd.latin1());
       uid_t u=getuid();
       cmd.sprintf("cat *.log >/tmp/klprfax.%d",u);
       system(cmd.latin1());
       cmd.sprintf("rm -f *");
       system(cmd.latin1());
       chdir("..");
       rmdir(dir.latin1());
       unlink(script.latin1());
       exit(k>>8);
     }
     else {

        chdir("..");

       // process a message loop for as long as forkResult exists
       // except when forkResult==-1

       if (forkResult==-1) {
         KMessageBox::error(this,i18n("Can't create child process to fax, your system maybe full"));
       }
       else {int status=-1;
         do {
           waitpid(forkResult,&status,WNOHANG);
           app->processEvents(50);
  				  if (!WIFEXITED(status)) { sleep(1); }
         }
         while(!WIFEXITED(status));
         k=WEXITSTATUS(status);

        if (k==0) {
           doing->setText("fax successfully sent");
           progress->setProgress(4);
         }
         else {QString txt,check=i18n("check the log in '/tmp/klprfax.");
           QString m;
           m.sprintf("%d'",getuid());
           check+=m;

           switch (k) {
             case 1 : txt=i18n("number busy or modem already in use");
             break;
             case 2 : txt=i18n("Is your modem connected? Did you SUID root efax?");
             break;
             case 3 : txt=i18n("modem error, maybe no fax at the other side?");
             break;
             case 4 : txt=i18n("no response from modem");
             break;
             case 5 : txt=i18n("program killed");
             break;
             default: txt=i18n("I can't figure out what happened");
           }
           txt=txt+"\n"+check;
           KMessageBox::error(this,txt);
           doing->setText(i18n("NO SUCCESS"));
           progress->setProgress(4);
         }

         goFax->setEnabled(true);
         noFax->setEnabled(true);
       }
     }
  }
}

/** No descriptions */
void Klprfax::findEFax()
{
int k;
const char *file="/tmp/klprfax";
char cmd[1024];
QString s;

   sprintf(cmd,"which fax >%s 2>&1",file);
   k=system(cmd);

   if (k==0) {
              FILE *f;
              char line[1024];

     f=fopen(file,"rt");
     fgets(line,1024,f);
     line[strlen(line)-1]='\0';
     s=(const char *) line;
     fclose(f);

   }
   else { FILE *f;
     f=fopen("/usr/bin/fax","rb");
     if (f!=NULL) { s="/usr/bin/fax";fclose(f); }
     else {
       f=fopen("/usr/local/bin/fax","rb");
       if (f!=NULL) { s="/usr/local/bin/fax";fclose(f); }
       else {
         KMessageBox::error(this,"Can't find 'efax' on this system, install EFAX");
         fatal=true;
       }
     }
   }

   unlink(file);

   efax=s;
}
/** No descriptions */
void Klprfax::doAbout()
{
  KAboutApplication(this).exec();
}
